﻿#!/bin/bash
# Installation script for itf2img.
#
# Copyright 2017 Mamoru Kaminaga
# Released under the MIT license
# http://opensource.org/licenses/mit-license.php
#
# This script must be called in the following way.
# ```
# sudo bash itf2img_install.sh
# ```
#
# Installation targets:
# * zlib 1.2.11
# * libpng 1.6.34
# * itf2img 1.0.0
#
# Updates:
# 2017/12/22 Mamoru Kaminaga
#  * The first version
#  * Target is itf2img version is v1.0.0
#  * Tested on Cygwin, MSYS2 on Windows 8.1 and Ubuntu 16.04

################################################################################
# Version dependent variables and processes
################################################################################
ZLIB='zlib-1.2.11'
ZLIB_URL="https://www.zlib.net/${ZLIB}.tar.gz"

LIBPNG='libpng-1.6.34'
LIBPNG_URL="https://download.sourceforge.net/libpng/${LIBPNG}.tar.gz"

ITF2IMG='itf2img_v1.0.0'
ITF2IMG_URL="http://yui.kz.tsukuba.ac.jp/wp-content/uploads/2017/12/${ITF2IMG}.zip"

zlib_patch_for_cygwin_and_msys() {
  # zlib patch for Cygwin or MSYS
  if [[ "${OS}" == 'Cygwin_or_MSYS' ]]; then
    echo "
    Patch for zlib for cygwin.
    The error for _wopen not found is avoided by applying this patch.
    "
    local ptch='1.2.11-cygwin-no-widechar.patch'
    mkdir patches
    wget -r -O patches/${ptch} https://raw.githubusercontent.com/Alexpux/MSYS2-packages/master/zlib/${ptch}
    if [[ $? != 0 ]]; then exit $?; fi
    patch -u < patches/${ptch}
    if [[ $? != 0 ]]; then exit $?; fi
  fi
}

################################################################################
# Check before run.
################################################################################
check_command() {
  which ${1} > /dev/null
  if [[ $? != 0 ]]; then
    echo "Error... Command ${1} is not installed"
    exit $?
  fi
}
check_command wget
check_command tar
check_command unzip
check_command make
check_command gcc

check_dir() {
  ls ${1} > /dev/null
  if [[ $? != 0 ]]; then
    echo "Error... Directory ${1} does not exist"
    exit $?
  fi
}
check_dir /usr/local/src
check_dir /usr/local/lib
check_dir /usr/local/include
check_dir /usr/local/bin

################################################################################
# Check the installation mode
################################################################################
echo "
Which installation behavior do you need?
  1: Install zlib, libpng and itf2img. All requirement is installed.
  2: Install libpng and itf2img. Pre-installed zlib is used.
  3: Install zlib and itf2img. Pre-installed libpng is used.
  4: Install only itf2img. Pre-installed zlib and libpng is used.
  5: cancel.

  If you don't want to overwrite pre-installed zlib and libpng, run this script
  in mode '4'. This enables you to use pre-installed zlib and libpng. In this
  cases, you may have to modify ${ITF2IMG}/makefile_unix.mk to change shared
  library path manually.
"
echo '>'
read MODE

INSTALL_ZLIB=0
INSTALL_LIBPNG=0
case ${MODE} in
  1) INSTALL_ZLIB=1; INSTALL_LIBPNG=1;;
  2) INSTALL_ZLIB=0; INSTALL_LIBPNG=1;;
  3) INSTALL_ZLIB=1; INSTALL_LIBPNG=0;;
  4) INSTALL_ZLIB=0; INSTALL_LIBPNG=0;;
  5) exit 0;;
  *) echo 'Invalid number'; exit 1;;
esac

################################################################################
# Check the OS
################################################################################
echo "
========================================
            Checking the OS
========================================
"

OS=''
if [[ "$(uname)" == 'Darwin' ]]; then
  OS='Mac'
  echo 'OS is MAC, not supported'
  exit 1
elif [[ "$(expr substr $(uname -s) 1 10)" == 'MINGW32_NT' ]] ||
  [[ "$(expr substr $(uname -s) 1 7)" == 'MSYS_NT' ]] ||
	[[ "$(expr substr $(uname -s) 1 9)" == 'CYGWIN_NT' ]]; then
  OS='Cygwin_or_MSYS'
elif [[ "$(expr substr $(uname -s) 1 5)" == 'Linux' ]]; then
  OS='Linux'
else
  echo 'OS is unknown, process as Linux? (yes/no)'
  echo '>'
  read INPUT
  if [[ ${INPUT} != "yes" ]]; then
    echo 'Stop installation'
    exit 0
  fi
  OS='Linux'
fi
echo "OS is ${OS}"

################################################################################
# Install zlib
################################################################################
if [[ ${INSTALL_ZLIB} -eq 1 ]]; then
  ls /usr/local/lib/libz.a
  if [[ $? -eq 0 ]]; then
    echo "
    zlib is already installed, do you want to overwrite it? (yes/no)
    "
    echo '>'
    read INPUT
    if [[ ${INPUT} != "yes" ]]; then
      echo 'Stop installation'
      exit 0
    fi
  fi

  echo "
========================================
            Installing zlib
========================================
  "

  wget -r -O /usr/local/src/${ZLIB}.tar.gz ${ZLIB_URL}
  if [[ $? != 0 ]]; then exit $?; fi
  cd /usr/local/src
  tar -zxvf ${ZLIB}.tar.gz
  cd ${ZLIB}
  zlib_patch_for_cygwin_and_msys
  ./configure
  if [[ $? != 0 ]]; then exit $?; fi
  make
  if [[ $? != 0 ]]; then exit $?; fi
  make install
fi

################################################################################
# Install libpng
################################################################################
if [[ ${INSTALL_LIBPNG} -eq 1 ]]; then
  ls /usr/local/lib/libpng.a
  if [[ $? -eq 0 ]]; then
    echo "
    libpng is already installed, do you want to overwrite it? (yes/no)
    "
    echo '>'
    read INPUT
    if [[ ${INPUT} != "yes" ]]; then
      echo 'Stop installation'
      exit 0
    fi
  fi

  echo "
========================================
          Installing libpng
========================================
  "

  wget -r -O /usr/local/src/${LIBPNG}.tar.gz ${LIBPNG_URL}
  if [[ $? != 0 ]]; then exit $?; fi
  cd /usr/local/src
  tar -zxvf ${LIBPNG}.tar.gz
  cd ${LIBPNG}

  if [[ "${OS}" == 'Cygwin_or_MSYS' ]]; then
    # Configure on Cygwin_or_MSYS
    # The following errors are avoided:
    # checking for zlibVersion in -lz... no
    # checking for yeszlibVersion in -lz... no
    # configure: error: zlib not installed
    CPPFLAGS="-I/usr/local/include" LDFLAGS="-L/usr/local/lib" ./configure
    if [[ $? != 0 ]]; then exit $?; fi
  else
    # Configure on Linux
    ./configure
    if [[ $? != 0 ]]; then exit $?; fi
  fi
  make
  if [[ $? != 0 ]]; then exit $?; fi
  make install
fi

################################################################################
# Install itf2img
################################################################################
echo "
========================================
          Installing itf2img
========================================
"

wget -r -O /usr/local/src/${ITF2IMG}.zip ${ITF2IMG_URL}
if [[ $? != 0 ]]; then exit $?; fi
cd /usr/local/src
unzip ${ITF2IMG}.zip
cd /usr/local/src/${ITF2IMG}
make -f makefile_unix.mk
if [[ $? != 0 ]]; then exit $?; fi

# The prefix is changed for Cygwin or MSYS.
if [[ "${OS}" == 'Cygwin_or_MSYS' ]]; then
  mv itf2img.out itf2img.exe
  install itf2img.exe /usr/local/bin
else
  mv itf2img.out itf2img
  install itf2img /usr/local/bin
fi

################################################################################
# Result
################################################################################
itf2img --version
if [[ $? != 0 ]]; then
  echo "Installation failed"
  exit $?;
else
  echo "Installation succeeded"
fi
